@@ -3,17 +3,22 @@ import csv
 import asyncio
 from sqlalchemy import create_engine, MetaData, Table
 from threading import RLock
+from .component import Component, AsyncComponent
+from typing import List, Union
 
 # from sqlalchemy.ext.asyncio import create_async_engine
 
-__all__ = ["CsvStore", "AsyncCsvStore", "DbStore"]
+__all__ = ["Store", "CsvStore", "AsyncCsvStore", "DbStore"]
 
 
-class Store(abc.ABC):
+class Store(Component):
     @abc.abstractmethod
     def store(self):
         ...
 
+    def __call__(self, *args, **kwargs):
+        return self.store(*args, **kwargs)
+
 
 class CsvStore(Store):
     def __init__(self, file_name):
@@ -27,38 +32,19 @@ class CsvStore(Store):
                 writer.writerow(items)
             return items
 
-    def __call__(self, *args, **kwargs):
-        return self.store(*args, **kwargs)
 
-    def __repr__(self):
-        return self.__class__.__name__
-
-    def __str__(self):
-        return self.__repr__()
-
-
-class AsyncCsvStore(Store):
+class AsyncCsvStore(Store, AsyncComponent):
     def __init__(self, file_name):
-        self.Async = ""
         self._filename = file_name
 
     async def store(self, items):
-        items = await items
-        fileds = list(items)
-        with open(self._filename, "a+", newline="") as csvfile:
-            writer = csv.DictWriter(csvfile, fieldnames=fileds)
-            with await asyncio.Lock():
-                writer.writerow(items)
-        return items
-
-    def __call__(self, *args, **kwargs):
-        return self.store(*args, **kwargs)
-
-    def __repr__(self):
-        return self.__class__.__name__
-
-    def __str__(self):
-        return self.__repr__()
+        if items:
+            fileds = list(items)
+            with open(self._filename, "a+", newline="") as csvfile:
+                writer = csv.DictWriter(csvfile, fieldnames=fileds)
+                async with asyncio.Lock():
+                    writer.writerow(items)
+            return items
 
 
 dblock = RLock()
@@ -72,22 +58,19 @@ class DbStore(Store):
         self.metadata = MetaData()
         self.metadata.reflect(bind=self.engine)
 
-    def store(self, items: dict):
-        dblock.acquire()
-        self.engine.execute(self.metadata.tables[self._table_name].insert(), items)
-        dblock.release()
-        return items
-
-    def __call__(self, *args, **kwargs):
-        return self.store(*args, **kwargs)
-
-    def __repr__(self):
-        return self.__class__.__name__
-
-    def __str__(self):
-        return self.__repr__()
+    def store(self, items: Union[dict, List[dict]]):
+        if items:
+            dblock.acquire()
+            self.engine.execute(self.metadata.tables[self._table_name].insert(), items)
+            dblock.release()
+            return items
 
 
-class AsyncDbStore(Store):
+class AsyncDbStore(Store, AsyncComponent):
     def __init__(self, connection_url=None, table_name=None):
-        self.Async = ""
+        ...
+
+
+class AsyncDbManyStore(Store, AsyncComponent):
+    def __init__(self, connection_url=None, table_name=None):
+        ...
