@@ -1,14 +1,21 @@
 from datetime import datetime
 from typing import List
-from functools import reduce
-from reprlib import repr
+import reprlib
 import importlib
 import sys
 import spydy
 from spydy.urls import Urls
-from spydy.defaults import RUNMODES, LEGAL_GLOBALS, LEGAL_RECOVERYS
-from spydy.adpaters import url_for_request
-from spydy.exceptions import UrlsStepNotFound
+from tabulate import tabulate
+from spydy.defaults import (
+    SPYDY_DEFUALT_SECTION_NAME,
+    RUNMODES,
+    LEGAL_GLOBALS,
+    LEGAL_RECOVERYS,
+    RECOVERY_TYPE,
+    VERBOSE,
+    LOG_TIME_FORMAT,
+)
+from spydy.exceptions import UrlsNotFound, StatsLogNotFound, ExceptionLogNotFound
 
 
 def class_dispatcher(user_provide_classname: str):
@@ -56,14 +63,10 @@ def get_class_from_moudle(module=None, value=None):  # packge class
 
 def get_value_from_moudle(*args, **kwargs):
     file_value = get_class_from_moudle(*args, **kwargs)
-    if callable(file_value):  # function without args, then call it
-        return file_value()
-    else:
-        return file_value
+    return file_value
 
 
 def parse_arguments(args: dict) -> dict:
-    # breakpoint()
     result_args = {}
     for k, v in args.items():
         if v.startswith("file:"):
@@ -86,31 +89,35 @@ def get_file_value(value):
     return get_value_from_moudle(module=user_package, value=user_value)
 
 
+def check_configs_and_add_defaults(configs):
+    check_configs(configs)
+    add_defaults(configs)
+
+
 def check_configs(configs):
     """
-    Check if the configs have the right forms
+    Configs Sanity checks
     """
     try:
         assert "PipeLine" in configs
     except AssertionError:
-        raise AssertionError("Section <PipeLine> seems not be inclued")
+        raise ValueError("Section <PipeLine> seems not be inclued")
     try:
         assert "Globals" in configs
     except AssertionError:
-        raise AssertionError("Section <Globals> seems not be inclued")
-    try:
-        assert (
-            "run_mode" in configs["Globals"]
-            and configs["Globals"]["run_mode"] in RUNMODES
-        )
-    except:
-        raise AssertionError(
-            "Parameter <run_mode> seems not be provides or run_mode not in {}".format(
-                RUNMODES
-            )
-        )
+        raise ValueError("Section <Globals> seems not be inclued")
 
+    check_run_mode(configs)
     global_arguments = configs["Globals"]
+    check_globals_has_legal_keys(global_arguments)
+
+    if "recovery_type" in global_arguments:
+        check_recovery_type_in_default_recoverys(global_arguments["recovery_type"])
+
+    check_custom_section_in_pipeline(configs)
+
+
+def check_globals_has_legal_keys(global_arguments):
     for argument in global_arguments:
         if argument not in LEGAL_GLOBALS:
             raise ValueError(
@@ -120,7 +127,21 @@ def check_configs(configs):
             )
 
 
-def check_recovery_type(recovery_type):
+def check_run_mode(configs):
+    try:
+        assert (
+            "run_mode" in configs["Globals"]
+            and configs["Globals"]["run_mode"] in RUNMODES
+        )
+    except:
+        raise ValueError(
+            "Parameter <run_mode> seems not be provides or run_mode not in {}".format(
+                RUNMODES
+            )
+        )
+
+
+def check_recovery_type_in_default_recoverys(recovery_type):
     if recovery_type not in LEGAL_RECOVERYS:
         raise ValueError(
             "{!r} not in allowed spydy, spydy only supports following recovery types: {}".format(
@@ -129,11 +150,47 @@ def check_recovery_type(recovery_type):
         )
 
 
-def linear_pipelinefunc(a, b):
-    if callable(a):
-        return b(a())
-    else:
-        return b(a)
+def check_custom_section_in_pipeline(configs):
+    section_names = list(configs)
+    right_sections = SPYDY_DEFUALT_SECTION_NAME + list(configs["PipeLine"])
+
+    for section in section_names:
+        if section not in right_sections:
+            raise ValueError(
+                "section name {!r} is wrong, please check if it really sit under [PipeLine] section.".format(
+                    section
+                )
+            )
+
+
+def add_defaults(configs):
+    add_verbose_default(configs)
+
+
+def add_verbose_default(configs):
+    if "verbose" not in configs["Globals"]:
+        configs["Globals"]["verbose"] = VERBOSE
+
+
+def get_verbose(configs):
+    try:
+        return bool(configs["Globals"]["verbose"])
+    except:
+        raise ValueError(
+            "Verbose setting in the [Globals] section should can not treated as Bool; Check if you give a right value of verbose"
+        )
+
+
+def get_interval(configs):
+    return (
+        int(configs["Globals"].get("interval"))
+        if configs["Globals"].get("interval", None)
+        else None
+    )
+
+
+def get_recovery_type(configs):
+    return configs["Globals"].get("recovery_type", RECOVERY_TYPE)
 
 
 def print_pipeline(pipeline: list):
@@ -141,29 +198,49 @@ def print_pipeline(pipeline: list):
     print(msg + "\n")
 
 
-def print_msg(msg, info_header="INFO", time_format="%Y-%m-%d %H:%M:%S", verbose=False):
-    msg = msg if verbose else repr(msg)
-    time_info = datetime.now().strftime(time_format)
+def print_msg(msg, info_header="INFO", time_format=LOG_TIME_FORMAT, verbose=False):
+    verbose = bool(verbose)
+    msg = msg if verbose else reprlib.repr(msg)
+    time_info = get_current_time_string(time_format)
     message = "|".join([info_header, time_info, msg])
     print(message)
 
 
-def get_step_from_pipeline(pipeline, step_type="urls"):
+def get_current_time_string(fmt):
+    return datetime.now().strftime(fmt)
+
+
+def get_step_from_pipeline(pipeline, step_type):
     """
-    get a step from pipeline by a given step_type
+    Get a step from pipeline by a given step_type
     """
     if step_type == "url":
         for step in pipeline:
             if isinstance(step, Urls):
                 return step
-        raise UrlsStepNotFound
-    if step_type == "statsLog":
+        raise UrlsNotFound
+    if step_type == "request":
+        from spydy.request import Request
+
+        for step in pipeline:
+            if isinstance(step, Request):
+                return step
+        return None
+    elif step_type == "statsLog":
         from spydy.logs import StatsReportLog
 
         for step in pipeline:
             if isinstance(step, StatsReportLog):
                 return step
-        return None  # StatsReportLog not found in pipeline
+        return None
+    elif step_type == "exceptionLog":
+        from spydy.logs import ExceptionLog
+
+        for step in pipeline:
+            if isinstance(step, ExceptionLog):
+                return step
+        return None
+
     else:
         raise StepTypeNotSupported
 
@@ -191,11 +268,10 @@ def get_config_ifexists(parser, section_name, setting_name):
 
 
 def handle_exceptions(
-    run_mode,
     temp_results,
-    pipleline: List,
+    pipeline: List,
     coroutine_id=None,
-    recovery_type="url_back_last",
+    recovery_type="url_back_end",
 ):
     """
     Handle exception during workflow by handle_type
@@ -207,14 +283,10 @@ def handle_exceptions(
       :pipeline: list of spydy pipeline components instance
       :handle_type: choose a way to deal with the exception when encountered an exception
     """
-    check_recovery_type(recovery_type)
-    if run_mode == "once":
-        url_step = get_step_from_pipeline(pipleline, step_type="url")
-        if hasattr(url_step, "handle_exception"):
-            url = get_temp_result(type(url_step), temp_results, coroutine_id)
-            url_step.handle_exception(
-                recovery_type=recovery_type, url=url_for_request(url)
-            )
+    url_step = get_step_from_pipeline(pipeline, step_type="url")
+    if hasattr(url_step, "handle_exception"):
+        url = get_temp_result(type(url_step), temp_results, coroutine_id)
+        url_step.handle_exception(recovery_type=recovery_type, url=url)
 
 
 def get_total_from_urls(urls_instance):
@@ -231,7 +303,7 @@ def get_total_from_urls(urls_instance):
         return None
 
 
-def convert_seconds_to_formal(seconds):
+def convert_seconds_to_standard_format(seconds):
     """
     Convert  seconds to a formal time format
     """
@@ -254,7 +326,7 @@ def convert_seconds_to_formal(seconds):
     return "{}h:{}m:{}s".format(H, M, S)
 
 
-def print_stats_log(stats: dict):
+def print_stats_log(stats: dict, add_time_info=True):
     """
     Print table-formatted infomations
 
@@ -262,11 +334,48 @@ def print_stats_log(stats: dict):
     :stats: A dict object
     """
     output = " {}: {}|" * len(stats)
-    output += "\r"
     infos = []
     for h, c in stats.items():
         infos.append(h)
         infos.append(c)
     info_table = output.format(*infos)
-    sys.stdout.write(info_table)
-    sys.stdout.flush()
+    if add_time_info:
+        info_table = get_current_time_string(LOG_TIME_FORMAT) + "|" + info_table
+    print(info_table)
+
+
+def wrap_exceptions_message(e):
+    full_message = repr(e)
+    len_of_message = len(full_message)
+    max_oneline_length = 40
+    num_msg_slices = len_of_message // max_oneline_length + 1
+    last_msg_length = len_of_message % max_oneline_length
+    msg_slices = []
+    for i in range(num_msg_slices):
+        if (i + 1) < num_msg_slices:
+            msg_slices.append(
+                full_message[i * max_oneline_length : (i + 1) * max_oneline_length]
+            )
+        else:
+            msg_slices.append(
+                full_message[
+                    i * max_oneline_length : (i * max_oneline_length + last_msg_length)
+                ]
+            )
+
+    return "\n".join(msg_slices)
+
+
+def print_table(infos: dict, add_time_info=True):
+    table_data = sorted(infos.items(), key=lambda item: item[1], reverse=True)
+    table_header = ["ErrorType", "Counts"]
+    if add_time_info:
+        print(get_current_time_string(fmt=LOG_TIME_FORMAT))
+    print(tabulate(table_data, headers=table_header, tablefmt="grid"))
+
+
+def run_if_callable(val):
+    if callable(val):
+        return val()
+    else:
+        return val
